@@ -1,8 +1,11 @@
+{-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE QuasiQuotes #-}
 {-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE OverloadedStrings #-}
 
+import           Control.Exception            (Exception)
 import           Control.Monad.IO.Class       (liftIO)
+import           Data.Typeable                (Typeable)
 import           Data.XML.Types
 import           Test.HUnit                   hiding (Test)
 import           Test.Hspec
@@ -23,6 +26,7 @@ import qualified Data.Set as Set
 import Control.Exception (toException)
 
 import qualified Data.Conduit as C
+import qualified Control.Monad.Trans.Resource as C
 import qualified Data.Conduit.List as CL
 import qualified Data.Map as Map
 import Text.Blaze (toMarkup)
@@ -85,6 +89,7 @@ main = hspec $ do
     describe "ordering attributes explicitly" $ do
         it "works" caseOrderAttrs
     it "parsing CDATA" caseParseCdata
+    it "retains namespaces when asked" caseRetainNamespaces
 
 documentParseRender :: IO ()
 documentParseRender =
@@ -301,13 +306,17 @@ cursorDeep = do
   (cursor $/ Cu.element "bar2" &/ Cu.element "baz2" >=> Cu.attribute "attr") @?= ["y"]
   null (cursor $| Cu.element "foo") @?= False
 cursorForce = do
-  Cu.force () [] @?= (Nothing :: Maybe Integer)
-  Cu.force () [1] @?= Just (1 :: Int)
-  Cu.force () [1,2] @?= Just (1 :: Int)
+  Cu.force DummyEx [] @?= (Nothing :: Maybe Integer)
+  Cu.force DummyEx [1] @?= Just (1 :: Int)
+  Cu.force DummyEx [1,2] @?= Just (1 :: Int)
 cursorForceM = do
-  Cu.forceM () [] @?= (Nothing :: Maybe Integer)
-  Cu.forceM () [Just 1, Nothing] @?= Just (1 :: Int)
-  Cu.forceM () [Nothing, Just (1 :: Int)] @?= Nothing
+  Cu.forceM DummyEx [] @?= (Nothing :: Maybe Integer)
+  Cu.forceM DummyEx [Just 1, Nothing] @?= Just (1 :: Int)
+  Cu.forceM DummyEx [Nothing, Just (1 :: Int)] @?= Nothing
+
+data DummyEx = DummyEx
+    deriving (Show, Typeable)
+instance Exception DummyEx
 
 showEq :: (Show a, Show b) => Either a b -> Either a b -> Assertion
 showEq x y = show x @=? show y
@@ -531,3 +540,23 @@ caseParseCdata = do
                     ])
                 []
     Res.parseLBS_ def lbs @?= doc
+
+caseRetainNamespaces :: Assertion
+caseRetainNamespaces = do
+    let lbs = "<foo xmlns:bar='baz'><bar:bin/><bin3 xmlns='bin4'></bin3></foo>"
+        doc = Res.parseLBS_ def { Res.psRetainNamespaces = True } lbs
+    doc `shouldBe` Res.Document
+        (Res.Prologue [] Nothing [])
+        (Res.Element
+            "foo"
+            (Map.singleton "xmlns:bar" "baz")
+            [ Res.NodeElement $ Res.Element
+                "{baz}bin"
+                Map.empty
+                []
+            , Res.NodeElement $ Res.Element
+                "{bin4}bin3"
+                (Map.singleton "xmlns" "bin4")
+                []
+            ])
+        []
